// ==============================================================
// $Id: A5.cpp,v 1.22 2006/01/25 19:49:21 dh219 Exp $
// CVE-lite compatible Ariane V by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================



/* $Id: A5.cpp,v 1.22 2006/01/25 19:49:21 dh219 Exp $ */

#include <cstdio>
#include <cmath>
#include "../../include/orbitersdk.h"
#include "cvel.h"
#include "initguidance.h"

PARTICLESTREAMSPEC srb_contrail = {
	0, 1.8, 3.0, 150.0, 0.1, 10, 12, 2.0, PARTICLESTREAMSPEC::DIFFUSE,
	PARTICLESTREAMSPEC::LVL_PLIN, 0.92, 1.8,
	PARTICLESTREAMSPEC::ATM_PLOG, 1e-10, 1e-10
};
PARTICLESTREAMSPEC srb_exhaust = {
	0, 1.8, 1000.0, 150.0, 0.1, 0.6, 12, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
	PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
};

/*
PARTICLESTREAMSPEC srb_exhaust = {
	0, 2.8, 1000.0, 150.0, 0.1, 0.6, 12, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
	PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
};
*/

/*#define SRBTHRUST 6474600.0
#define SRBTHRUST_SL 5260000.0
#define SRBISP_SL 2452.5
#define SRBISP_V 2697.8
*/
class A5: public COMMON_VESSEL_EX {
public:
	A5 (OBJHANDLE hVessel, int FlightModel);
	~A5 ();
	void classcaps();
	void loadstate( FILEHANDLE scn, void *vs );
	void savestate( FILEHANDLE scn );
	void timestep( double simt );
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount );
	int consumekey( const char *keystate );
	
	void setbooststage();
	void setmainstage();
	void setupperstage();
	void addthrusters();

	void blowboosters();
	void blowcore();
	void blowfairing();
	void setstatus( int );

protected:
	void thrustvec();
	void srb_thrust( double time );
	double main_engine_level();
	
	int status;
	double m_fair;

	VECTOR3 OFF_LHB;
	VECTOR3 OFF_RHB;
	VECTOR3 OFF_STG1;
	VECTOR3 OFF_STG2;
	VECTOR3 OFF_LHF;
	VECTOR3 OFF_RHF;

	MESHHANDLE h_msh_lhf;
	MESHHANDLE h_msh_rhf;
	MESHHANDLE h_msh_lhf_l;
	MESHHANDLE h_msh_rhf_l;
	MESHHANDLE h_msh_lhb;
	MESHHANDLE h_msh_rhb;
	MESHHANDLE h_msh_stg1;
	MESHHANDLE h_msh_stg2;
	UINT msh_lhb ;
	UINT msh_rhb ;
	UINT msh_stg1 ;
	UINT msh_lhf ;
	UINT msh_rhf ;
	PROPELLANT_HANDLE ph_main;
	PROPELLANT_HANDLE ph_second;
	PROPELLANT_HANDLE ph_booster;
	THRUSTER_HANDLE th_main[1];
	THRUSTER_HANDLE th_second[1];
	THRUSTER_HANDLE th_lhb[1];
	THRUSTER_HANDLE th_rhb[1];
	THRUSTER_HANDLE th_pup[2];
	THRUSTER_HANDLE th_pdown[2];
	THRUSTER_HANDLE th_yleft[2];
	THRUSTER_HANDLE th_yright[2];
	THRUSTER_HANDLE th_bleft[2];
	THRUSTER_HANDLE th_bright[2];
	THRUSTER_HANDLE th_retro[2];
	THRUSTER_HANDLE th_fwd[2];
	SURFHANDLE tex;
	double detachmainstage;
	double fair_z;
	bool fair_large;
	double t_maxq;
	double oldq;
	double met;
	double oldsimt;
	bool killmains;

	VECTOR3 comshift;
	VECTOR3 VZERO;
	double UPPERMASS, SRBMASS, COREMASS;

	IGUIDE *iguide;

	double userheight;

	VISHANDLE vishandle;

	bool A5G, A5ECA, A5ATV;

	double flevel[3];

	double SRBTHRUST;

private:
	void setoffsets();
	bool initialsetup;
	bool fair_attached;
	bool booster_attached;
	bool core_attached;
	bool coreignited;
	bool metactive;
	bool boosterignite;
	bool enginesdead;
	bool autothrottle;
	bool contrail;
	void VulcainThrust();

};


void maxabs( double *in, const double max );


A5 *a5 = NULL;

/* API BITS */

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	a5 = new A5( hvessel, flightmodel );
	return (VESSEL*)a5;
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	A5 *ariane5;
	if (vessel) {
		ariane5 = static_cast<A5*>(vessel);
		delete ariane5;
	}
}
DLLCLBK void ovcSetClassCaps( VESSEL *vessel, FILEHANDLE cfg )
{
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	ariane5->classcaps();	
}
DLLCLBK void ovcTimestep (VESSEL *vessel, double simt )
{
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	ariane5->timestep( simt );
}
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	return( ariane5->consumekey( keystate ) );
}
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)
{
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	ariane5->loadstate( scn, vs );
}
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	ariane5->savestate( scn );
}
DLLCLBK void ovcVisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) {
	A5 *ariane5;
	ariane5 = static_cast<A5*>(vessel);
	ariane5->VisualCreated( vessel, vis, refcount );
}
/**********************/





/* A5 CLASS */

A5::A5(OBJHANDLE hVessel, int FlightModel) : COMMON_VESSEL_EX( hVessel ) {

	A5G = false;
	A5ATV = false;
	A5ECA = false;

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "EC" ) )
		A5ECA = true;
	else if( !strncmp( classname, "A5E", 3 ) || strstr( classname, "ATV" ) )
		A5ATV = true;
	else
		A5G = true;


	flevel[0] = 1.0;
	flevel[1] = 1.0;
	flevel[2] = 1.0;

	initialsetup = false;

	status = 2;
	m_fair = 0.0;
	
	comshift = _V(0,0,0);
	VZERO=_V(0,0,0);

	msh_lhb = 999;
	msh_rhb = 999;
	msh_stg1 = 999;
	msh_lhf = 999;
	msh_rhf = 999;

	oldsimt = 0.0;

	met = 0.0;
	metactive = false;
	oldq = 0.0;
	t_maxq = -1.0;

	h_msh_lhf_l = oapiLoadMesh( "ariane5\\fairup_fairing_1" );
	h_msh_rhf_l = oapiLoadMesh( "ariane5\\fairup_fairing_2" );

	h_msh_lhf = oapiLoadMesh( "ariane5\\fairup2_fairing_1" );
	h_msh_rhf = oapiLoadMesh( "ariane5\\fairup2_fairing_2" );


	h_msh_lhb = oapiLoadMesh( "ariane5\\booster_EAP_1" );
	h_msh_rhb = oapiLoadMesh( "ariane5\\booster_EAP_2" );

	h_msh_stg1 = oapiLoadMesh( "ariane5\\core" );

if( A5G ) {
	h_msh_stg2 = oapiLoadMesh( "ariane5\\upper" );
	UPPERMASS = 1275.0;
	SRBMASS = 39800.0;
	SRBTHRUST = 5886000;
}
if( A5ATV ) {
	h_msh_stg2 = oapiLoadMesh( "ariane5\\upper" );
	UPPERMASS = 1275.0;
	SRBMASS = 38200.0;
	SRBTHRUST = 6376500;
}
if( A5ECA ) {
	h_msh_stg2 = oapiLoadMesh( "ariane5\\upper_ecb" );
	UPPERMASS = 2100.0;
	SRBMASS = 38200.0;
	SRBTHRUST = 6376500;
}

	COREMASS = 12500.0;
	


	
	fair_z = 0.0;
	detachmainstage = 0.0;
	fair_attached = false;
	core_attached = false;
	booster_attached = false;

	coreignited = false;
	boosterignite = false;
	killmains = false;

	tex = oapiRegisterExhaustTexture ("Exhaust2");

	enginesdead = false;

	OFF_LHB = _V(0.0,0.0,0.0);
	OFF_RHB = _V(0.0,0.0,0.0);
	OFF_STG1 = _V(0.0,0.0,0.0);
	OFF_STG2 = _V(0.0,0.0,0.0);
	OFF_LHF = _V(0.0,0.0,0.0);
	OFF_RHF = _V(0.0,0.0,0.0);

	autothrottle = false;

	MeshVisibilityPreference(MESHVIS_ALWAYS);

	iguide = new IGUIDE();

	userheight = 0.0;

	vishandle = NULL;

	th_main[0] = NULL;

	contrail = false;

}
A5::~A5() {

}

void A5::classcaps(){
	

	th_lhb[0] = NULL;
	th_rhb[0] = NULL;
	ph_booster = NULL;
	ph_main = NULL;
	ph_second = NULL;
	



if( A5G ) {
	ph_second = CreatePropellantResource( 9800.00 );
	ph_main = CreatePropellantResource( 159000.0 );
	ph_booster = CreatePropellantResource( 238000.0 * 2.0 );
}
if( A5ATV ) {
	ph_second = CreatePropellantResource( 10050.0 );
	ph_main = CreatePropellantResource( 173300.0 );
	ph_booster = CreatePropellantResource( 261800.0 * 2.0 );
}
if( A5ECA ) {
	ph_second = CreatePropellantResource( 14700.0 );
	ph_main = CreatePropellantResource( 173300.0 );
	ph_booster = CreatePropellantResource( 261800.0 * 2.0 );
}

	SetDefaultPropellantResource( ph_main );



}
void A5::loadstate( FILEHANDLE scn, void *vs  ){

    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
        if (!strnicmp (line, "CONFIGURATION", 13)) {
            sscanf (line+13, "%d", &status);
		}
		else if(!strnicmp (line, "FAIRING", 7) ) {
			fair_attached = true;
			char tmp[80];
			sscanf( line+7, "%s %lf", &tmp, &fair_z );
			unsigned int j;
			for( j = 0 ; j < strlen( tmp ) ; j++ )
				tmp[j] = tolower(tmp[j]);
			if( strstr( tmp, "large" ) )
				fair_large = true;
			else
				fair_large = false;
		}		
		else if( !strnicmp( line, "MET", 3 ) ) {
			sscanf( line+3, "%lf", &met );
			metactive = true;
		}
		else if( !strnicmp( line, "INITIALHEADING", 14 ) ) {
			double ihead;
			sscanf( line+14, "%lf", &ihead );
			iguide->SetUserHeading(ihead);
			iguide->Active(true);
		}
		else if( !strnicmp( line, "HEIGHT", 6 ) ) {
			sscanf( line+6, "%lf", &userheight );
		}
		else if( !strnicmp( line, "CONTRAIL", 9 ) ) {
			contrail= true;
		}
		else {
			if( !strnicmp( line, "PRPLEVEL", 8 ) ) {
				char *ptr;
				if( ( ptr = strstr( line, "0:" ) ) )
					flevel[0] = atof( ptr+2 );
				if( ( ptr = strstr( line, "1:" ) ) )
					flevel[1] = atof( ptr+2 );
				if( ( ptr = strstr( line, "2:" ) ) )
					flevel[2] = atof( ptr+2 );
			}
			
			ParsePayloadString( line );

			ParseScenarioLineEx (line, vs);

		}		
	}
	
	setstatus(status);
	// removed 29/10/04 to try to prevent reload crashes.
}
void A5::savestate( FILEHANDLE scn ){
	char cbuf[256];

	SaveDefaultState (scn);

	oapiWriteScenario_int (scn, "CONFIGURATION", status );
	if( fair_attached ) {
		if( fair_large )
			sprintf( cbuf, "LARGE %2.2f", fair_z );
		else
			sprintf( cbuf, "SMALL %2.2f", fair_z );
		oapiWriteScenario_string (scn, "FAIRING", cbuf );
	}
	if( iguide->Active() ) {
		oapiWriteScenario_float(scn, "INITIALHEADING", iguide->GetUserHeading() );
	}
	if( metactive ) {
		oapiWriteScenario_float( scn, "MET", met );
	}
	if( contrail ) {
		oapiWriteScenario_int( scn, "CONTRAIL", 1 );
	}


	SaveDefaultStateEx(scn);	
}
int A5::consumekey( const char *keystate ) {

	if (KEYMOD_SHIFT (keystate)) {
		// reserved for MFDs
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				if( fair_attached ) {
					blowfairing();
				}
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_J, 0.5)) {
				VESSEL *v;
				if( status == 2 && !fair_attached ) {
					v = DeployPayload();
					oapiSetFocusObject(v->GetHandle());
					setstatus(status);
					return 1;
				}
			}
		}
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				if( status < 2 )
					setstatus(status+1);
				else if( !fair_attached ) {
					DeployPayload(_V(0,0,0)-comshift);
					setstatus(status);
				}
				return 1;
			}
		}
	}

	return 0;

}
void A5::timestep( double simt ) {

	double dt;
	dt = simt - oldsimt;
	

	if( initialsetup == false ) {
		setstatus(status);
	}


	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
		setstatus(status);
	}

	if( th_main[0] ) {
		if( ( status == 0 && GetThrusterLevel( th_main[0] ) > 0.01 ) && !metactive ) {
			metactive = true;
			met = -7.0;
			autothrottle = true;
		}
	}

	if( metactive == true ) {
		met += dt;
		if( met > 0.0 && status <= 1 && coreignited && GetThrusterLevel(th_main[0]) < 0.29 ) {
			coreignited = false;
			killmains = true;
		}
		
		if( met > -7.0 && status <= 1 ) {
			coreignited = true;
//			VulcainThrust();
		}

		if( met > 0.0 && status == 0 ) {
			boosterignite = true;
			srb_thrust( met );
//	SetThrusterLevel(th_lhb[0],1.0);
//	SetThrusterLevel(th_rhb[0],1.0);
		}

		if( status == 0 && ph_booster ) {	
			if( GetPropellantMaxMass(ph_booster) > 0.001 ) {
				double fill = GetPropellantMass(ph_booster)/GetPropellantMaxMass(ph_booster);
				if( fill < 0.01 && oapiGetTimeAcceleration() > 1.0 )
					oapiSetTimeAcceleration(1.0);
				if( fill < 0.001 && GetDynPressure() < 30000.0 ) {
					boosterignite = false;
					setstatus(1);
				}
			}
		}
		if( status == 1 && GetPropellantMass(ph_main)/GetPropellantMaxMass(ph_main) < 0.01 ) {
			setstatus(2);
		}
		if( status <= 1 && coreignited && GetThrusterGroupLevel( THGROUP_MAIN ) < 0.29 ) {
			SetThrusterGroupLevel( THGROUP_MAIN, 0.0 );
			DelThrusterGroup( THGROUP_MAIN );
		}


	}


	if( fair_attached && status > 0 && GetAltitude() > 30000.0 && ( 0.5 * GetAtmDensity() * pow( GetAirspeed(), 3.0 ) ) < 1135.0 ) {
		blowfairing();
	}




	if( met > 129600.0 && enginesdead == false ) {
		ClearExhaustRefs();
		ClearAttExhaustRefs();
  		ClearThrusterDefinitions();
		enginesdead = true;
	}

	

	thrustvec();


	if( status == 0 && GroundContact() ) {
		if( GetPropellantMass(ph_booster)/GetPropellantMaxMass(ph_booster) > flevel[0] )
			SetPropellantMass(ph_booster,GetPropellantMaxMass(ph_booster)*flevel[0] );
		if( GetPropellantMass(ph_main)/GetPropellantMaxMass(ph_main) > flevel[1] )
			SetPropellantMass(ph_main,GetPropellantMaxMass(ph_main)*flevel[1] );
		if( GetPropellantMass(ph_second)/GetPropellantMaxMass(ph_second) > flevel[2] )
			SetPropellantMass(ph_second,GetPropellantMaxMass(ph_second)*flevel[2] );
	}

	
	oldsimt = simt;

	if( metactive ) {
		sprintf( oapiDebugString(), "MET: %3.0f", met );
	}

}

void A5::setstatus( int st ) {

	status = st;
	initialsetup = true;


	if( fair_attached )
		m_fair = 3000.0;
	else
		m_fair = 0.0;
		

	switch( st ){
	case(0):
		setbooststage();
		break;
	case(1):
		setmainstage();
		break;
	case(2):
		setupperstage();
		break;
	}



}
void A5::setbooststage() {

	COMMON_MESH tmp;

	ClearMeshesEx();


	/*
	SetEmptyMass( COREMASS + SRBMASS + SRBMASS + UPPERMASS + m_fair + CalculateMass() );
	ShiftCentreOfMass (VZERO-comshift);
	comshift = _V(0,0,-15.0);
	ShiftCentreOfMass (comshift);
*/


	VECTOR3 v_com;
	double plmass;
	double aumass;

	plmass = CalculateMass(&v_com);

	aumass = COREMASS + SRBMASS + SRBMASS + UPPERMASS + m_fair + plmass;
	SetEmptyMass( aumass );

	ShiftCentreOfMass (VZERO-comshift);
	comshift.x = (v_com.x * plmass)/(aumass);
	comshift.y = (v_com.y * plmass)/(aumass);
	comshift.z = -15.0 + (v_com.z * plmass)/(aumass);
	ShiftCentreOfMass (comshift);
	

	SetSize(50);
	SetEnableFocus( true );
	SetPitchMomentScale (1e-7);
	SetBankMomentScale (1e-7);
	SetLiftCoeffFunc (0); 
	SetSurfaceFrictionCoeff( 0.8, 0.8 );
	SetRotDrag (_V(0.7,0.7,0.3));


	
	SetTouchdownPoints( _V( 1.0, 0.0, -40.0-userheight )-comshift, _V( -1.0, -1.0, -40.0-userheight )-comshift, _V( -1.0, 1.0, -40.0-userheight )-comshift );
	SetCrossSections( _V( 167.0, 167.0, 30.0 ) );
	SetPMI (_V(100.0, 100.0, 7.0 ));
	SetCameraOffset( _V( 0.0, 5.0, -20.0)-comshift );
	SetCW(0.09, 0.09, 1.4, 1.4);



	setoffsets();

	if( fair_attached ) {
		if( fair_large ) {
			tmp.MeshHandle( h_msh_lhf_l );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf_l );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
		else {
			tmp.MeshHandle( h_msh_lhf );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
	}


	tmp.MeshHandle( h_msh_lhb );
	tmp.MeshPosition( OFF_LHB );
	AddMeshEx( tmp );

	tmp.MeshHandle( h_msh_rhb );
	tmp.MeshPosition( OFF_RHB );
	AddMeshEx( tmp );

	tmp.MeshHandle( h_msh_stg1 );
	tmp.MeshPosition( OFF_STG1 );
	AddMeshEx( tmp );

	tmp.MeshHandle( h_msh_stg2 );
	tmp.MeshPosition( OFF_STG2 );
	AddMeshEx( tmp );


	Draw(_V(0,0,0)-comshift);

	double thruster, srbs;
	thruster = GetThrusterGroupLevel( THGROUP_MAIN );
	if( th_lhb[0] )
		srbs = GetThrusterLevel( th_lhb[0] );
	else
		srbs = 0.0;
	addthrusters();
	SetThrusterGroupLevel( THGROUP_MAIN, thruster );
	SetThrusterLevel( th_lhb[0], srbs );
	SetThrusterLevel( th_rhb[0], srbs );

	booster_attached = true;
	core_attached = true;

}
void A5::setmainstage() {

	COMMON_MESH tmp;

	ClearMeshesEx();	

/*	double aumass;

	aumass = COREMASS + UPPERMASS + m_fair + CalculateMass();
	SetEmptyMass( aumass );

	ShiftCentreOfMass (VZERO-comshift);
	comshift = _V(0,0,-9.0 + 19.0 * (CalculateMass()/aumass) );
	ShiftCentreOfMass (comshift);
*/
	

	VECTOR3 v_com;
	double plmass;
	double aumass;

	plmass = CalculateMass(&v_com);

	aumass = COREMASS + UPPERMASS + m_fair + plmass;
	SetEmptyMass( aumass );

	ShiftCentreOfMass (VZERO-comshift);
	comshift.x = (v_com.x * plmass)/(aumass);
	comshift.y = (v_com.y * plmass)/(aumass);
	comshift.z = -10.0 + (v_com.z * plmass)/(aumass);
	ShiftCentreOfMass (comshift);




	SetSize(50);
	SetEnableFocus( true );
	SetPitchMomentScale (1e-7);
	SetBankMomentScale (1e-7);
	SetLiftCoeffFunc (0); 
	SetSurfaceFrictionCoeff( 0.8, 0.8 );
	SetRotDrag (_V(0.7,0.7,0.3));

	SetTouchdownPoints( _V( -1.0, -2.5, -10.0 ), _V( 0.0, -2.5, 10.0 ), _V( 1.0, -2.5, -10.0 ) );
	SetCrossSections( _V( 140.0, 140.0, 20.0 ) );
	SetPMI (_V(80.0, 80.0, 3.0 ));
	SetCameraOffset( _V( 0.0, 5.0, -20.0)-comshift );
	SetCW(0.09, 0.09, 1.4, 1.4);

	setoffsets();

	tmp.MeshHandle( h_msh_stg2 );
	tmp.MeshPosition( OFF_STG2 );
	AddMeshEx( tmp );

	if( fair_attached ) {
		if( fair_large ) {
			tmp.MeshHandle( h_msh_lhf_l );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf_l );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
		else {
			tmp.MeshHandle( h_msh_lhf );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
	}

	tmp.MeshHandle( h_msh_stg1 );
	tmp.MeshPosition( OFF_STG1 );
	AddMeshEx( tmp );

	Draw(_V(0,0,0)-comshift);
	
	double thruster;
	thruster = GetThrusterLevel( th_main[0] );

	if( booster_attached == true ) {
		blowboosters();
		addthrusters();
		SetThrusterLevel(th_main[0], 1.0 );
	}
	else {
		addthrusters();
	}


	if( ph_booster ) {
		SetPropellantMaxMass(ph_booster,0.0);
	}


	if( !killmains )
		SetThrusterGroupLevel( THGROUP_MAIN, thruster );
	else
		SetThrusterGroupLevel( THGROUP_MAIN, 0.0 );
	
	booster_attached = false;
	core_attached = true;

}
void A5::setupperstage(){

	COMMON_MESH tmp;

	ClearMeshesEx();	

	VECTOR3 v_com;
	double plmass;
	double aumass;

	plmass = CalculateMass(&v_com);

	aumass = UPPERMASS + m_fair + plmass;
	SetEmptyMass( aumass );

	ShiftCentreOfMass (VZERO-comshift);
	comshift.x = (v_com.x * plmass)/(aumass);
	comshift.y = (v_com.y * plmass)/(aumass);
	comshift.z = (v_com.z * plmass)/(aumass);
	ShiftCentreOfMass (comshift);

	SetSize(15);
	SetEnableFocus( true );
	SetPitchMomentScale (-1e-5);
	SetBankMomentScale (-1e-5);
	SetLiftCoeffFunc (0); 
	SetSurfaceFrictionCoeff( 0.8, 0.8 );
	SetRotDrag (_V(0.7,0.7,0.3));

	SetTouchdownPoints( _V( -1.0, -1.0, -2.0)-comshift, _V( 0.0, 1.0, -2.0)-comshift , _V( 1.0, -1.0, -2.0)-comshift );
	SetCrossSections( _V( 40.0, 40.0, 20.0 ) );
	SetPMI (_V(2.0+90,2.0+90, 2.0+45 ));
	SetCameraOffset( _V( 0.0, 6.0, 5.0)-comshift );
	SetCameraOffset( _V( 0.0, 5.0, -2.0)-comshift );
	SetCW(0.09, 0.09, 1.4, 1.4);

	setoffsets();	




	tmp.MeshHandle( h_msh_stg2 );
	tmp.MeshPosition( OFF_STG2 );
	AddMeshEx( tmp );


	if( fair_attached ) {
		if( fair_large ) {
			tmp.MeshHandle( h_msh_lhf_l );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf_l );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
		else {
			tmp.MeshHandle( h_msh_lhf );
			tmp.MeshPosition( OFF_LHF );
			AddMeshEx( tmp );
			tmp.MeshHandle( h_msh_rhf );
			tmp.MeshPosition( OFF_RHF );
			AddMeshEx( tmp );
		}
	}

	Draw(_V(0,0,0)-comshift);


	if( core_attached == true ) {
		blowcore();
	}
	
	if( ph_booster ) {
		SetPropellantMass( ph_booster, 0.0 );
		SetPropellantMaxMass( ph_booster, 0.0 );
	}
	if( ph_main ) {
		SetPropellantMass( ph_main, 0.0 );
		SetPropellantMaxMass( ph_main, 0.0 );
	}
	SetDefaultPropellantResource( ph_second );
	
	double thruster;
	thruster = GetThrusterGroupLevel( THGROUP_MAIN );

	addthrusters();
	
	SetThrusterGroupLevel( THGROUP_MAIN, thruster );


}

void A5::addthrusters() {

	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();

	th_lhb[0] = NULL;
	th_rhb[0] = NULL;
	th_main[0] = NULL;
	th_second[0] = NULL;

	PROPELLANT_HANDLE ph_utility;


	if( status <= 2 ) {

		if( status < 2 )
			ph_utility = ph_main;
		else
			ph_utility = ph_second;

		
		th_pdown[0] = CreateThruster( _V(0.0,-2.0,0.0)-comshift, _V(0.0,0.0,-1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_pdown[1] = CreateThruster( _V(0.0,2.0,0.0)-comshift, _V(0.0,0.0,1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_pup[0] = CreateThruster( _V(0.0,-2.0,0.0)-comshift, _V(0.0,0.0,1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_pup[1] = CreateThruster( _V(0.0,2.0,0.0)-comshift, _V(0.0,0.0,-1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_yright[0] = CreateThruster( _V(-2.0,0.0,0.0)-comshift, _V(0.0,0.0,1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_yright[1] = CreateThruster( _V(2.0,0.0,0.0)-comshift, _V(0.0,0.0,-1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_yleft[0] = CreateThruster( _V(2.0,0.0,0.0)-comshift, _V(0.0,0.0,1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_yleft[1] = CreateThruster( _V(-2.0,0.0,0.0)-comshift, _V(0.0,0.0,-1.0), 400.0, ph_utility, 3000.0, 0.0 ) ;

		
		th_bleft[0] = CreateThruster( _V(-3.0,0.0,0.0)-comshift, _V(0.0,-1.0,0.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_bright[0] = CreateThruster( _V(-3.0,0.0,0.0)-comshift, _V(0.0,1.0,0.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_bleft[1] = CreateThruster( _V(3.0,0.0,0.0)-comshift, _V(0.0,1.0,0.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		th_bright[1] = CreateThruster( _V(3.0,0.0,0.0)-comshift, _V(0.0,-1.0,0.0), 400.0, ph_utility, 3000.0, 0.0 ) ;
		CreateThrusterGroup( th_bleft, 2, THGROUP_ATT_BANKLEFT );
		CreateThrusterGroup( th_bright, 2, THGROUP_ATT_BANKRIGHT );

		CreateThrusterGroup( th_pup, 2, THGROUP_ATT_PITCHUP );	
		CreateThrusterGroup( th_pdown, 2, THGROUP_ATT_PITCHDOWN );	
		CreateThrusterGroup( th_yleft, 2, THGROUP_ATT_YAWLEFT );	
		CreateThrusterGroup( th_yright, 2, THGROUP_ATT_YAWRIGHT );	

		th_retro[0] = th_yright[1];
		th_retro[1] = th_yleft[1];

		th_fwd[0] = th_yright[0];
		th_fwd[1] = th_yleft[0];

		CreateThrusterGroup( th_retro, 2, THGROUP_ATT_BACK );
		CreateThrusterGroup( th_fwd, 2, THGROUP_ATT_FORWARD );

if( A5G ) {
		th_second[0] = CreateThruster( OFF_STG2+_V(0.0,0.0,-2.0)-comshift, _V(0.0,0.0,1.0), 29000.0, ph_second, 3178.0, 1.0, 1e5 );
		AddExhaust( th_second[0], 1.5, 0.5 );
		CreateThrusterGroup( th_second, 1, THGROUP_MAIN );
}
if( A5ATV ) {
		th_second[0] = CreateThruster( OFF_STG2+_V(0.0,0.0,-2.0)-comshift, _V(0.0,0.0,1.0), 29000.0, ph_second, 3178.0, 1.0, 1e5 );
		AddExhaust( th_second[0], 1.5, 0.5 );
		CreateThrusterGroup( th_second, 1, THGROUP_MAIN );
}
if( A5ECA ) {
		th_second[0] = CreateThruster( OFF_STG2+_V(0.0,0.0,-1.0)-comshift, _V(0.0,0.0,1.0), 64746.0, ph_second, 4375.0, 1.0, 1e5 );
		AddExhaust( th_second[0], 3.0, 0.5 );
		CreateThrusterGroup( th_second, 1, THGROUP_MAIN );
}


	}	
	
	if( status <= 1  ) {


if( A5G ) {
		th_main[0] = CreateThruster( _V(0.0,0.0,-31.5)-comshift, _V(0.0,0.0,1.0), 1128150.0, ph_main, 4218.3, 3286.0, 1e5 ) ;
		CreateThrusterGroup( th_main, 1, THGROUP_MAIN );
		AddExhaust( th_main[0], 3.0, 0.5 );
}
if( A5ATV ) {
		th_main[0] = CreateThruster( _V(0.0,0.0,-31.5)-comshift, _V(0.0,0.0,1.0), 1275300.0, ph_main, 4257.5, 3286.0, 1e5 ) ;
		CreateThrusterGroup( th_main, 1, THGROUP_MAIN );
		AddExhaust( th_main[0], 3.0, 0.5 );
}
if( A5ECA ) {
		th_main[0] = CreateThruster( _V(0.0,0.0,-31.5)-comshift, _V(0.0,0.0,1.0), 1275300.0, ph_main, 4257.5, 3286.0, 1e5 ) ;
		CreateThrusterGroup( th_main, 1, THGROUP_MAIN );
		AddExhaust( th_main[0], 3.0, 0.5 );
}
	
	}
	if( status == 0 ) {



if( A5G ) {
		th_lhb[0] = CreateThruster( _V( -4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ), SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
		th_rhb[0] = CreateThruster( _V( 4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ),  SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
}
if( A5ATV ) {
		th_lhb[0] = CreateThruster( _V( -4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ), SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
		th_rhb[0] = CreateThruster( _V( 4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ),  SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
}
if( A5ECA ) {
		th_lhb[0] = CreateThruster( _V( -4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ), SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
		th_rhb[0] = CreateThruster( _V( 4.2, 0.0, -32.5)-comshift , _V( 0.0, 0.0, 1.0 ),  SRBTHRUST, ph_booster, 2570.2, 2000.2, 1e5 );
}

		
		AddExhaust( th_lhb[0], 15.0, 1.0, tex );
		AddExhaust( th_rhb[0], 15.0, 1.0, tex );
		AddExhaustStream(th_lhb[0], &srb_exhaust);
		AddExhaustStream(th_rhb[0], &srb_exhaust);
		if( contrail ) {
			AddExhaustStream(th_lhb[0], &srb_contrail);
			AddExhaustStream(th_rhb[0], &srb_contrail);
		}

	
	}

}
void A5::blowboosters() {

	VESSELSTATUS vs, vsl, vsr;
	char name[256];
	OBJHANDLE h1, h2;
	VESSEL *v1, *v2;
	PROPELLANT_HANDLE ph_lhb_aux, ph_rhb_aux;
	THRUSTER_HANDLE th_lhb_aux, th_rhb_aux;

	GetStatus(vs);


	VECTOR3 lhb_rot;
	VECTOR3 rhb_rot;
	VECTOR3 lhb_rvel;
	VECTOR3 rhb_rvel;

	lhb_rot=_V( 0.0, 0.1, 0.0 );
	rhb_rot=_V( 0.0, 0.1, 0.0 );
	

	GlobalRot( _V( -8.0, 0.0, 0.0 ), lhb_rvel );
	GlobalRot( _V( 8.0, 0.0, 0.0 ), rhb_rvel );

	vsl = vs;
	vsl.vrot.x += lhb_rot.x;
	vsl.vrot.y += lhb_rot.y;
	vsl.vrot.z += lhb_rot.z;

	vsl.rvel.x += lhb_rvel.x;
	vsl.rvel.y += lhb_rvel.y;
	vsl.rvel.z += lhb_rvel.z;

	vsr = vs;
	vsr.vrot.x += rhb_rot.x;
	vsr.vrot.y += rhb_rot.y;
	vsr.vrot.z += rhb_rot.z;

	vsr.rvel.x += rhb_rvel.x;
	vsr.rvel.y += rhb_rvel.y;
	vsr.rvel.z += rhb_rvel.z;

	sprintf( name, "%s-LHB", GetName() );
	Local2Rel( OFF_LHB-comshift, vsl.rpos );
	h1 = oapiCreateVessel( name, "ariane5\\ariane5_eap", vsl );
	v1 = oapiGetVesselInterface( h1 );

	sprintf( name, "%s-RHB", GetName() );
	Local2Rel( OFF_RHB-comshift, vsr.rpos );
	vsr.arot.z += PI;
	h2 = oapiCreateVessel( name, "ariane5\\ariane5_eap", vsr );
	v2 = oapiGetVesselInterface( h2 );

	ph_lhb_aux = v1->CreatePropellantResource( GetPropellantMass(ph_booster)/2.0, GetPropellantMass(ph_booster)/2.0, 1.0 );
	th_lhb_aux = v1->CreateThruster( _V(0.0, 0.0, -15.0), _V( 0.0, 0.0, 1.0 ), GetThrusterMax(th_lhb[0]), ph_lhb_aux, GetThrusterIsp( th_lhb[0] ),0.0, 0.0);

	v1->SetThrusterLevel( th_lhb_aux, GetThrusterLevel( th_lhb[0] ) );


	ph_rhb_aux = v2->CreatePropellantResource( GetPropellantMass(ph_booster)/2.0, GetPropellantMass(ph_booster)/2.0, 1.0 );
	th_rhb_aux = v2->CreateThruster( _V(0.0, 0.0, -15.0), _V( 0.0, 0.0, 1.0 ),  GetThrusterMax(th_rhb[0]), ph_rhb_aux, GetThrusterIsp( th_rhb[0] ),0.0, 0.0);
	v2->SetThrusterLevel( th_rhb_aux, GetThrusterLevel( th_rhb[0] ) );

	v1->AddExhaust( th_lhb_aux, 15.0, 1.0, tex );
	v2->AddExhaust( th_rhb_aux, 15.0, 1.0, tex );

	v1->AddExhaustStream(th_lhb_aux, &srb_exhaust );
	v2->AddExhaustStream(th_rhb_aux, &srb_exhaust );
	if( contrail ) {
		v1->AddExhaustStream(th_lhb_aux, &srb_contrail );
		v2->AddExhaustStream(th_rhb_aux, &srb_contrail );
	}

	boosterignite = false;
}
void A5::blowcore() {

	VESSELSTATUS vs;
	char name[256];
	OBJHANDLE h1;
	VESSEL *v1;
	PROPELLANT_HANDLE ph_main_aux;
	THRUSTER_HANDLE th_main_aux1;
	THRUSTER_HANDLE th_main_aux2;
	THRUSTER_HANDLE th_main_aux3;
	THRUSTER_HANDLE th_main_aux4;

	GetStatus(vs);

	sprintf( name, "%s-STG1", GetName() );
	Local2Rel( OFF_STG1-comshift, vs.rpos );
	h1 = oapiCreateVessel( name, "ariane5\\ariane5_epc", vs );

	v1 = oapiGetVesselInterface( h1 );
	
	ph_main_aux = v1->CreatePropellantResource( 500.0, 500.0, 1.0 );
	th_main_aux1 = v1->CreateThruster( _V(-2.0,0.0,15.0), _V(0.0,0.0,-1.0), 10000.0, ph_main_aux, 3351.0, 2800.0, 1e5 ) ;
	th_main_aux2 = v1->CreateThruster( _V(2.0,0.0,15.0), _V(0.0,0.0,-1.0), 10000.0, ph_main_aux, 3351.0, 2800.0, 1e5 ) ;
	th_main_aux3 = v1->CreateThruster( _V(0.0,-2.0,15.0), _V(0.0,0.0,-1.0), 10000.0, ph_main_aux, 3351.0, 2800.0, 1e5 ) ;
	th_main_aux4 = v1->CreateThruster( _V(0.0,2.0,15.0), _V(0.0,0.0,-1.0), 10000.0, ph_main_aux, 3351.0, 2800.0, 1e5 ) ;

	v1->SetThrusterLevel( th_main_aux1, 1.0 );
	v1->SetThrusterLevel( th_main_aux2, 1.0 );
	v1->SetThrusterLevel( th_main_aux3, 1.0 );
	v1->SetThrusterLevel( th_main_aux4, 1.0 );
	v1->AddExhaust( th_main_aux1, 1.0, 0.3);
	v1->AddExhaust( th_main_aux2, 1.0, 0.3);
	v1->AddExhaust( th_main_aux3, 1.0, 0.3);
	v1->AddExhaust( th_main_aux4, 1.0, 0.3);

	core_attached = false;

}
void A5::blowfairing() {

	VESSELSTATUS vs;
	char name[256];
	OBJHANDLE h1, h2;
	VESSEL *v1, *v2;
	PROPELLANT_HANDLE ph_lhf_aux, ph_rhf_aux;
	THRUSTER_HANDLE th_lhf_aux, th_rhf_aux;


	if( fair_attached ) {

		GetStatus(vs);

		fair_attached = false;
		sprintf( name, "%s-LHF", GetName() );
		Local2Rel( OFF_LHF-comshift, vs.rpos );
		if( fair_large )
			h1 = oapiCreateVessel( name, "ariane5\\ariane5_lhf", vs );
		else
			h1 = oapiCreateVessel( name, "ariane5\\ariane5_lhf2", vs );

		sprintf( name, "%s-RHF", GetName() );
		Local2Rel( OFF_RHF-comshift, vs.rpos );
		if( fair_large )
			h2 = oapiCreateVessel( name, "ariane5\\ariane5_rhf", vs );
		else
			h2 = oapiCreateVessel( name, "ariane5\\ariane5_rhf2", vs );

		v1 = oapiGetVesselInterface( h1 );
		v2 = oapiGetVesselInterface( h2 );
		
		ph_lhf_aux = v1->CreatePropellantResource( 7.5, 7.5, 1.0 );
		th_lhf_aux = v1->CreateThruster( _V(1.0, 0.0, 0.5), _V( -1.0, 0.0, 0.0 ), 450000.0, ph_lhf_aux, 3000.0, 0.0, 0 );
		v1->SetThrusterLevel( th_lhf_aux, 1.0 );

		ph_rhf_aux = v2->CreatePropellantResource( 7.5, 7.5, 1.0 );
		th_rhf_aux = v2->CreateThruster( _V(-1.0, 0.0, 0.5), _V( 1.0, 0.0, 0.0 ), 450000.0, ph_rhf_aux, 3000.0, 0.0, 0 );
		v2->SetThrusterLevel( th_rhf_aux, 1.0 );

		fair_attached = false;
		setstatus(status);
	}
}
void A5::thrustvec() {

	VECTOR3 avel;
	VECTOR3 dir;

	if( status == 0 ) {
		iguide->LaunchGuidance(this);
	}
	
	double pup = 0.01 * pow(GetThrusterGroupLevel( THGROUP_ATT_PITCHDOWN ),4.0);
	pup += -0.01 * pow(GetThrusterGroupLevel( THGROUP_ATT_PITCHUP ), 4.0);

	double yleft = 0.01 * pow(GetThrusterGroupLevel( THGROUP_ATT_YAWLEFT), 4.0);
	yleft += -0.01 * pow(GetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT ), 4.0);

	double rleft = 0.02 * pow( GetThrusterGroupLevel( THGROUP_ATT_BANKLEFT ), 4.0 );
	rleft += -0.02 * pow(GetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT ), 4.0);


	if( GetNavmodeState(NAVMODE_KILLROT) ) {
		GetAngularVel(avel);

		pup = pow( avel.x*180/PI, 3 )/ 5	;
		maxabs( &pup, 0.01 );

		yleft = -1.0 * pow ( avel.y*180/PI, 3 )/ 5 ;
		maxabs( &yleft, 0.01 );

		rleft = pow( avel.z*180/PI, 3 )/ 5;
		maxabs( &rleft, 0.02 );
			
	}


	if( status <= 1 && th_main[0] ) {
		SetThrusterDir( th_main[0], _V( yleft, pup, sqrt(1-pup*pup-yleft*yleft) ) );
		if( status == 0 ) {
			dir = _V( yleft, -1.0 * rleft + pup, 1.0 );
			normalise(dir);
			SetThrusterDir( th_lhb[0], dir );
			dir.y = rleft + pup;
			SetThrusterDir( th_rhb[0], dir  );
		}		
	}
	else if( status == 2 && th_second[0] ) {
		pup *= 20;
		yleft *= 20;
		SetThrusterDir( th_second[0], _V( yleft, pup, sqrt(1-pup*pup-yleft*yleft) ) );
	}


}
void A5::srb_thrust( double time ) {

	
	if( !th_lhb[0] || !th_rhb[0] || !ph_booster )
		return;

	if( GetPropellantMaxMass(ph_booster) < 0.001 )
		return;

//	double maxthrust = ( SRBTHRUST_SL - SRBTHRUST ) * GetAtmPressure() / 1e5 + SRBTHRUST ;
	double maxthrust = SRBTHRUST;

	double scale;

	// dip the thrust quadratically between these times to 0.55 the total.

#define DIPSTART 40
#define DIPEND 70

	
	if( time < 10 )
		scale = 1;
	else if( time > DIPSTART && time < DIPEND )
		scale = 0.4 * pow( (time-(DIPEND+DIPSTART)/2.0)/(DIPEND-DIPSTART) , 2 ) + 0.55;
	else
		scale = 1.0;


	double fuelratio = GetPropellantMass(ph_booster) / GetPropellantMaxMass(ph_booster);
	if( fuelratio < 0.04 ) {
		scale = 100 * fuelratio ;
		maxabs( &scale, 1.0 );
	}

	SetThrusterMax0(th_lhb[0],maxthrust*scale);
	SetThrusterMax0(th_rhb[0],maxthrust*scale);

	if( th_lhb[0] && th_rhb[0] ) {
		double level;
		if( time < 0.0 ) {
			SetThrusterLevel( th_lhb[0], 0.0 );
			SetThrusterLevel( th_rhb[0], 0.0 );
		}
		else if( time > 0.0 && time < 0.3 ) {
			level = time / 0.3;
			SetThrusterLevel( th_lhb[0], level );
			SetThrusterLevel( th_rhb[0], level );
		}
		else if( time > 0.3 ) {
			SetThrusterLevel( th_lhb[0], 1.0 );
			SetThrusterLevel( th_rhb[0], 1.0 );
		}
	}




	return;

}
void A5::setoffsets() {

	double fudge;
	fudge = 0.0;
	


	OFF_LHB = _V( -4.0, 0.0, -16.0-fudge );
	OFF_RHB = _V( 4.0, 0.0, -16.0-fudge );
	OFF_STG1 = _V( 0.0, -0.2, -16.0-fudge );

	OFF_STG2 = _V( 0.0, 0.0, 0.0-fudge );

	if( fair_large ) {
		OFF_LHF = _V( -1.345, 0.0, 9.4+fair_z-fudge );
		OFF_RHF = _V(  1.345, 0.0, 9.4+fair_z-fudge );
	}
	else {
		OFF_LHF = _V( -1.345, 0.0, 7.5+fair_z-fudge );
		OFF_RHF = _V(  1.345, 0.0, 7.5+fair_z-fudge );
	}

}

double A5::main_engine_level() {
	
	double level;
	if( metactive ) {
		if( met >= -7.0 && met < 3.0 ) {
			level = 1.75 + met/4.0;
		}
		else if( met > -3.0 )
			level = 1.0;
	}
	else {
		level =0.0;
	}
	/*
	if( th_main[0] ) {
		if( metactive )
		return GetThrusterLevel(th_main[0]);
	}
	else
		return 0.0;*/

	return level;
}

void A5::VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) {
	vishandle = vis;
	setstatus(status);
}

/* Simon Proud's Vulcain Thrust Calc */
/* deprecated */
void A5::VulcainThrust() {

	double gammav;//new constants
	double cpresv;
	double epresv;
	double apres;
	double rconst;
	double ctempv;
	double atmweightv;
	double tareav;
	double eareav;
	double massflowv;
	double exvelv;
	double thrustvalv;//end of new constants


	gammav		=	1.19;		//ratio of specific heats for LOX/LOH
	cpresv		=	11600000;	//Chamber pressure
	epresv		=	38200;		//Exhaust pressure (not verified)
	rconst		=	8.314;		//Gas constant
	ctempv		=	3100;		//Chamber temperature
	atmweightv	=	0.0133;		//Weight of propellant
	tareav		=	0.0593;		//Throat area
	eareav		=	3.4693;		//Exhaust area
	//next line calculates mass flow for the vulcain-2
	massflowv	=	sqrt(gammav)*pow((2/(gammav+1)),0.5*(gammav+1)/(gammav-1))*cpresv*tareav/sqrt(rconst*ctempv/atmweightv);
	//next line calculates exhaust velocity for the vulcain-2
	exvelv		=	sqrt((2*gammav*rconst*ctempv/((gammav-1)*atmweightv))*(1-pow((epresv/cpresv),(gammav-1)/gammav)));


	apres=GetAtmPressure();
	thrustvalv=(massflowv*exvelv)+eareav*(epresv-apres);


	SetThrusterMax0(th_main[0],thrustvalv);
	SetThrusterIsp(th_main[0],thrustvalv/massflowv);


	double level = main_engine_level();
	if( killmains )
		level = 0.0;
	SetThrusterLevel( th_main[0], level  );

//	sprintf( oapiDebugString(), "F: %2.2f I_sp: %2.2f", thrustvalv, thrustvalv/massflowv );
	
}							



/***********************/

void maxabs( double *in, const double max ) {
	if( fabs(*in) > fabs(max) ) {
		*in = *in * fabs(max) / fabs(*in);
	}
}