// ==============================================================
// $Id: atv.cpp,v 1.9 2004/11/19 12:02:32 dh219 Exp $
// ATV by D Henderson orbiter@aibs.org.uk / C3PO
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include "cvel.h"

/*
PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.01, 1000, 2, 0, 0.05, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.1, 1000, 10, 0, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
*/

PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.05, 3000, 5, 0.1, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.4, 3000, 150, 0.05, 0.5, 5, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};


class ATV: public COMMON_VESSEL_EX {
public:
	ATV (OBJHANDLE hVessel, int FlightModel);
	~ATV ();
	void classcaps();
	void loadstate( FILEHANDLE scn, void *vs );
	void savestate( FILEHANDLE scn );
	void timestep( double simt );
	int consumekey( const char *keystate );
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount );
	
	void setmainstage();
	void addthrusters();

protected:
	int status;

	VECTOR3 OFF_ATV;

	MESHHANDLE h_msh_atv;
	UINT msh_atv ;
	PROPELLANT_HANDLE ph_main;
	THRUSTER_HANDLE th_main[8];
	THRUSTER_HANDLE th_back[8];
	THRUSTER_HANDLE th_fwd[4];
	THRUSTER_HANDLE th_up[4];
	THRUSTER_HANDLE th_down[4];
	THRUSTER_HANDLE th_left[4];
	THRUSTER_HANDLE th_right[4];

	THRUSTER_HANDLE th_pup[4];
	THRUSTER_HANDLE th_pdown[4];
	THRUSTER_HANDLE th_yleft[4];
	THRUSTER_HANDLE th_yright[4];
	THRUSTER_HANDLE th_bleft[4];
	THRUSTER_HANDLE th_bright[4];

	double comshift;
	double anim_deploy[30];
	UINT anim[30];
	UINT meshindex[30];
	bool fueldump;

private:
	void DefineAnimations();
	void makeanimation( int i, UINT groups[], int n, VECTOR3 ref, VECTOR3 axis, double range, double start, double end, int parent = -1);

	double animlevel;
	MGROUP_ROTATE *rot[30];
	bool animactive;
	UINT mshid;

	ANIMATIONCOMPONENT_HANDLE animcomp[30];
	
};

ATV *atvptr = NULL;

/* API BITS */

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	atvptr = new ATV( hvessel, flightmodel );
	return (VESSEL*)atvptr;
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	ATV *atv;
	if (vessel) {
		atv = static_cast<ATV*>(vessel);
		delete atv;
	}
}
DLLCLBK void ovcSetClassCaps( VESSEL *vessel, FILEHANDLE cfg )
{
	ATV *atv;
	atv = static_cast<ATV*>(vessel);
	atv->classcaps();	
}
DLLCLBK void ovcTimestep (VESSEL *vessel, double simt )
{
	ATV *atv;
	atv = static_cast<ATV*>(vessel);
	atv->timestep( simt );
}
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	ATV *atv;
	atv = static_cast<ATV*>(vessel);
	return( atv->consumekey( keystate ) );
}
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)
{
	ATV *atv;
	atv = static_cast<ATV*>(vessel);
	atv->loadstate( scn, vs );
}
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	ATV *atv;
	atv = static_cast<ATV*>(vessel);
	atv->savestate( scn );
}
DLLCLBK void ovcVisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	ATV *atv = (ATV*)vessel;
	atv->VisualCreated ( vessel,  vis,  refcount);
}
	
/**********************/


/* ATV Class */

ATV::ATV(OBJHANDLE hVessel, int FlightModel) : COMMON_VESSEL_EX( hVessel ) {

	status = 0;

	comshift = -0.4;
	OFF_ATV = _V( 0.0, 0.0, 0.0 );

	msh_atv = 0;
	ph_main = NULL;;

	int i;
	for( i = 0 ; i < 4 ; i++ ) {
		th_main[i] = NULL;
		th_back[i] = NULL;
		th_back[i+4] = NULL;
		th_fwd[i] = NULL;
		th_up[i] = NULL;
		th_down[i] = NULL;
		th_left[i] = NULL;
		th_right[i] = NULL;
		th_pup[i] = NULL;
		th_pdown[i] = NULL;
		th_yleft[i] = NULL;
		th_yright[i] = NULL;
		th_bleft[i] = NULL;
		th_bright[i] = NULL;
	}

	for( i = 0 ; i < 30 ; i++ ) {
		anim_deploy[i] = 0.0;
		anim[i] = 999;
		animcomp[i] = NULL;
	}
	animlevel = 0.0;
	animactive = false;

	fueldump = false;

	h_msh_atv = oapiLoadMesh("ariane5\\atv");

	mshid = 999;

	anim[0] = CreateAnimation (0.0);
	
}
ATV::~ATV()
{ }

void ATV::classcaps(){
	
	ph_main = CreatePropellantResource( 4700.0, -1.0, 1.0 );
	SetDefaultPropellantResource( ph_main );

	SetSize(10);
	SetEnableFocus( true );
	SetPitchMomentScale (-1e-5);
	SetBankMomentScale (-1e-5);
	SetLiftCoeffFunc (0); 
	SetSurfaceFrictionCoeff( 0.8, 0.8 );
	SetRotDrag (_V(1.5,1.5,0.5));
	SetTouchdownPoints( _V( -2.0, -2.0, -2.0-comshift ), _V( 0.0, -2.0, 2.0 ), _V( 2.0, -2.0, -2.0 ) );
	SetCrossSections( _V(93.0,93.0,63.9) );
	SetPMI (_V(6.1,6.1,2.24));
	SetCameraOffset( _V( -1.0, 3.0, 1.0-comshift ) );
	SetCW(2.2, 2.2, 2.2, 2.2);
	EnableTransponder(true);

	CreateDock(_V(0.0,0.0,4.45-comshift), _V(0,0,1), _V(0,1,0) );

//	setmainstage();
}

void ATV::loadstate( FILEHANDLE scn, void *vs  ){

    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
        if (!strnicmp (line, "PANELS", 6 )) {
            sscanf (line+6, "%lf", &animlevel);
			animactive = true;
		}
		else {
			ParseScenarioLineEx (line, vs);
			ParsePayloadString(line);
		}
	}

	setmainstage();
}
void ATV::savestate( FILEHANDLE scn ){
	SaveDefaultState (scn);
	oapiWriteScenario_float( scn, "PANELS", animlevel );
	SaveDefaultStateEx( scn );

}
int ATV::consumekey( const char *keystate ) {

	if (KEYMOD_SHIFT (keystate)) {
		// reserved for MFDs
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_F)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_F, 1.0)) {
				if( fueldump ) {
					SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
					SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
					fueldump = false;
				}
				else
					fueldump = true;
				return 1;
			}
		}
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) { // "Jettison Stage"
			if(oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) {
				if( animlevel < 1.0 )
					animactive = true;
				return 1;
			}
		}
	}

	return 0;

}
void ATV::timestep( double simt ) {


	if( animactive == true && anim[0] != 999 ) {
		animlevel += oapiGetSimStep() / 60.0;
		if( animlevel > 1.0 ) {
			animlevel = 1.0;
			animactive = false;
		}
		SetAnimation( anim[0], animlevel );
	}

	if( 0.5 * GetAtmDensity() * pow( GetAirspeed(), 3.0 ) > 8e7 ) {

		SetEnableFocus(false);
		OBJHANDLE focus, us;
		focus = oapiGetFocusObject();
		us = GetHandle();
		if( focus == us ) {
			if( oapiGetVesselCount() > 1 ) {
				int i;
				for( i = 0 ; i< oapiGetVesselCount() ; i++ ) {
					if( oapiGetVesselByIndex(i) != us ) {
						oapiSetFocusObject( oapiGetVesselByIndex(i) );
						break;
					}
				}
			}
		}
		focus = oapiGetFocusObject();
		if( focus != us ) {
			oapiDeleteVessel( us );
			return;
		}
	}

	if( fueldump ) {
		SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 1.0 );
		SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 1.0 );
	}

 
}

void ATV::setmainstage() {

	ClearMeshesEx();	

	SetEmptyMass( 16050.0 + CalculateMass() );


	mshid = AddMesh(h_msh_atv);
	SetMeshVisibilityMode(mshid,MESHVIS_ALWAYS);
	MeshVisibilityPreference(MESHVIS_ALWAYS);
	Draw();	

	
	DefineAnimations();


	addthrusters();

}

void ATV::addthrusters() {

	int i;

	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();	

	th_main[0] = CreateThruster( _V(0.861,0.819,-4.659-comshift), _V(0,0,1), 490, ph_main, 3100, 0, 0 );
	th_main[1] = CreateThruster( _V(-0.861,0.819,-4.659-comshift), _V(0,0,1), 490, ph_main, 3100, 0, 0 );
	th_main[2] = CreateThruster( _V(0.861,-0.819,-4.659-comshift), _V(0,0,1), 490, ph_main, 3100, 0, 0 );
	th_main[3] = CreateThruster( _V(-0.861,-0.819,-4.659-comshift), _V(0,0,1), 490, ph_main, 3100, 0, 0 );

/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_main[i], 4, 0.2 );
*/
	th_back[0] = CreateThruster( _V(-1.73,1.585,-3.46-comshift), _V(1,-1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[1] = CreateThruster( _V(-1.585,1.73,-3.46-comshift), _V(1,-1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[2] = CreateThruster( _V(1.585,1.73,-3.46-comshift), _V(-1,-1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[3] = CreateThruster( _V(1.73,1.585,-3.46-comshift), _V(-1,-1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[4] = CreateThruster( _V(-1.73,-1.585,-3.46-comshift), _V(1,1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[5] = CreateThruster( _V(-1.585,-1.73,-3.46-comshift), _V(1,1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[6] = CreateThruster( _V(1.585,-1.73,-3.46-comshift), _V(-1,1,-1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_back[7] = CreateThruster( _V(1.73,-1.585,-3.46-comshift), _V(-1,1,-1), 220.0 ,ph_main, 3100, 0, 0) ;

/*	for( i = 0 ; i < 8 ; i++ )
		AddExhaust( th_back[i], 0.5, 0.05 );
*/
	th_fwd[0] = CreateThruster( _V(-1.55,1.55,-3.966-comshift), _V(0,0,1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_fwd[1] = CreateThruster( _V(1.55,1.55,-3.966-comshift), _V(0,0,1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_fwd[2] = CreateThruster( _V(-1.55,-1.55,-3.966-comshift), _V(0,0,1), 220.0 ,ph_main, 3100, 0, 0) ;
	th_fwd[3] = CreateThruster( _V(1.55,-1.55,-3.966-comshift), _V(0,0,1), 220.0 ,ph_main, 3100, 0, 0) ;

/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_fwd[i], 0.5, 0.05 );
*/
	th_up[0] = CreateThruster( _V(-0.084,-2.25,2.874-comshift), _V(0,1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_up[1] = CreateThruster( _V(0.076,-2.25,2.874-comshift), _V(0,1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_up[2] = CreateThruster( _V(-1.365,-1.84,-3.676-comshift), _V(0,1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_up[3] = CreateThruster( _V(1.365,-1.84,-3.676-comshift), _V(0,1,0), 220.0 ,ph_main, 3100, 0, 0) ;

/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_up[i], 0.5, 0.05 );
*/
	th_down[0] = CreateThruster( _V(-0.084,2.25,2.874-comshift), _V(0,-1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_down[1] = CreateThruster( _V(0.076,2.25,2.874-comshift), _V(0,-1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_down[2] = CreateThruster( _V(-1.365,1.84,-3.676-comshift), _V(0,-1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_down[3] = CreateThruster( _V(1.365,1.84,-3.676-comshift), _V(0,-1,0), 220.0 ,ph_main, 3100, 0, 0) ;
	
/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_down[i], 0.5, 0.05 );
*/
	th_left[0]  = CreateThruster( _V(2.25,0.084,2.874-comshift), _V(-1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_left[1]  = CreateThruster( _V(2.25,-0.076,2.874-comshift), _V(-1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_left[2]  = CreateThruster( _V(1.84,1.365,-3.676-comshift), _V(-1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_left[3]  = CreateThruster( _V(1.84,-1.365,-3.676-comshift), _V(-1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;

/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_left[i], 0.5, 0.05 );
*/
	th_right[0]  = CreateThruster( _V(-2.25,0.084,2.874-comshift), _V(1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_right[1]  = CreateThruster( _V(-2.25,-0.076,2.874-comshift), _V(1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_right[2]  = CreateThruster( _V(-1.84,1.365,-3.676-comshift), _V(1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;
	th_right[3]  = CreateThruster( _V(-1.84,-1.365,-3.676-comshift), _V(1,0,0), 220.0 ,ph_main, 3100, 0, 0) ;

/*	for( i = 0 ; i < 4 ; i++ )
		AddExhaust( th_right[i], 0.5, 0.05 );
*/
	
	th_pup[0] = th_up[0];
	th_pup[1] = th_up[1];
	th_pup[2] = th_down[2];
	th_pup[3] = th_down[3];

	th_pdown[0] = th_up[2];
	th_pdown[1] = th_up[3];
	th_pdown[2] = th_down[0];
	th_pdown[3] = th_down[1];

	th_yleft[0] = th_left[0];
	th_yleft[1] = th_left[1];
	th_yleft[2] = th_right[2];
	th_yleft[3] = th_right[3];

	th_yright[0] = th_left[2];
	th_yright[1] = th_left[3];
	th_yright[2] = th_right[0];
	th_yright[3] = th_right[1];

	th_bleft[0] = th_left[2];
	th_bleft[1] = th_right[3];
	th_bleft[2] = th_up[3];
	th_bleft[3] = th_down[2];

	th_bright[0] = th_left[3];
	th_bright[1] = th_right[2];
	th_bright[2] = th_up[2];
	th_bright[3] = th_down[3];

	th_main[4] = th_fwd[0];
	th_main[5] = th_fwd[1];
	th_main[6] = th_fwd[2];
	th_main[7] = th_fwd[3];

	CreateThrusterGroup( th_main, 8, THGROUP_MAIN );	
	CreateThrusterGroup( th_back, 8, THGROUP_RETRO );	

	CreateThrusterGroup( th_back, 8, THGROUP_ATT_BACK );
	CreateThrusterGroup( th_fwd, 4, THGROUP_ATT_FORWARD );
	CreateThrusterGroup( th_up, 4, THGROUP_ATT_UP );
	CreateThrusterGroup( th_down, 4, THGROUP_ATT_DOWN );
	CreateThrusterGroup( th_right, 4, THGROUP_ATT_RIGHT );
	CreateThrusterGroup( th_left, 4, THGROUP_ATT_LEFT );	

	CreateThrusterGroup( th_yleft, 4, THGROUP_ATT_YAWLEFT );	
	CreateThrusterGroup( th_yright, 4, THGROUP_ATT_YAWRIGHT );	
	CreateThrusterGroup( th_bleft, 4, THGROUP_ATT_BANKLEFT );	
	CreateThrusterGroup( th_bright, 4, THGROUP_ATT_BANKRIGHT );	
	CreateThrusterGroup( th_pup, 4, THGROUP_ATT_PITCHUP );	
	CreateThrusterGroup( th_pdown, 4, THGROUP_ATT_PITCHDOWN );	

	for( i = 0 ; i < 4 ; i++ ) {
		AddExhaustStream(th_main[i], &oms_exhaust);
		AddExhaustStream(th_right[i], &rcs_exhaust);
		AddExhaustStream(th_up[i], &rcs_exhaust);
		AddExhaustStream(th_down[i], &rcs_exhaust);
		AddExhaustStream(th_left[i], &rcs_exhaust);
		AddExhaustStream(th_back[i], &rcs_exhaust);
		AddExhaustStream(th_back[i+4], &rcs_exhaust);
		AddExhaustStream(th_fwd[i], &rcs_exhaust);
	}
}

void ATV::DefineAnimations() {
	
	int i;
	for( i = 0 ; i < 30 ; i++ ) {
		if( animcomp[i] ) {
			DelAnimationComponent(anim[0], animcomp[i] );
			animcomp[i] = NULL;
		}
	
	}
	// big sticky thing
	
	static UINT groups0[1] = { 26 };
	makeanimation( 0, groups0 , 1, _V( 0,2.31,0.263-comshift ), _V(1,0,0 ), -0.5*PI, 0.2, 0.36 );

	// top right foldy out thing

	static UINT groups1[1] = { 34 };
//	makeanimation( 1, groups1, 1, _V(1.982,0.768,-3.823-comshift), _V(-0.3587,0.93358,0), 0.5*PI, 0, 0.1 );
	makeanimation( 1, groups1, 1, _V(1.982,0.768,-2.112-comshift), _V(-0.3587,0.93358,0), -0.5*PI, 0.05, 0.9 );

	static UINT groups2[4] = { 33 };

//	makeanimation( 2, groups2 , 1, _V(1.982,0.768,-1.87-comshift ), _V( -0.3587,0.93358,0), -0.5*PI, 0.1, 0.2, 1 );
	makeanimation( 2, groups2 , 1, _V(1.982,0.768,-3.823-comshift ), _V( -0.3587,0.93358,0), PI, 0.05, 0.9, 1 );

	static UINT groups3[1] = { 32 };
	makeanimation( 3, groups3 , 1, _V(1.982,0.768,-1.87-comshift ), _V(-0.3587,0.93358,0 ), -PI, 0.05, 0.9, 2);

	static UINT groups4[1] = { 31 };
	makeanimation( 4, groups4 , 1, _V(1.982,0.768,-3.823-comshift ), _V(-0.3587,0.93358,0 ), PI, 0.05, 0.9, 3 );

	static UINT groups5[1] = { 30 };
	makeanimation( 5, groups5 , 1, _V(1.982,0.768,-1.87-comshift ), _V( -0.3587,0.93358,0), -PI, 0.05, 0.9, 4 );

/*	static UINT groups6[1] = { 30  };
	makeanimation( 6, groups6 , 1, _V( 9.049,3.481,-2.112-comshift), _V( -0.3587,0.93358,0), 1.0*PI, 0.68, 0.87 );
*/
	static UINT groups7[5] = { 34,33,32,31,30 };
	makeanimation( 7, groups7 , 5, _V(1.982,0.768,-2.112-comshift ), _V( 0.93358,0.3587,0), 0.35*PI, 0.95, 1 );

	// bottom right foldy out thing

	static UINT groups8[1] = { 47 };
	makeanimation( 8, groups8, 1, _V(1.982,-0.768,-2.112-comshift), _V(0.3587,0.93358,0), -0.5*PI, 0.04, 0.89 );

	static UINT groups9[1] = { 51 };
	makeanimation( 9, groups9 , 1, _V(1.982,-0.768,-3.823-comshift ), _V( 0.3587,0.93358,0), PI, 0.04, 0.89, 8 );

	static UINT groups10[1] = { 50 };
	makeanimation( 10, groups10 , 1, _V( 1.982,-0.768,-1.87-comshift), _V( 0.3587,0.93358,0), -PI, 0.04, 0.89, 9 );

	static UINT groups11[1] = { 49 };
	makeanimation( 11, groups11 , 1, _V(1.982,-0.768,-3.823-comshift), _V( 0.3587,0.93358,0), PI, 0.04, 0.89, 10 );

	static UINT groups12[1] = { 48 };
	makeanimation( 12, groups12 , 1, _V( 1.982,-0.768,-1.87-comshift), _V( 0.3587,0.93358,0), -PI, 0.04, 0.89, 11 );

/*	static UINT groups13[1] = { 51  };
	makeanimation( 13, groups13 , 1, _V( 9.049,-3.481,-2.112-comshift), _V( 0.3587,0.93358,0), 1.0*PI, 0.68+0.05, 0.87+0.05 );
*/
	static UINT groups14[5] = { 47,48,49,50,51 };
	makeanimation( 14, groups14 , 5, _V(1.982,-0.768,-2.112 -comshift), _V( 0.93358,-0.3587,0), 0.35*PI, 0.89, 0.99 );

	// top left foldy out thing

	static UINT groups15[1] = { 35 };
	makeanimation( 15, groups15, 1, _V(-1.982,0.768,-2.112-comshift), _V(0.3587,0.93358,0), 0.5*PI, 0.06, 0.91 );

	static UINT groups16[1] = { 39 };
	makeanimation( 16, groups16 , 1, _V(-1.982,0.768,-3.823-comshift ), _V( 0.3587,0.93358,0), -PI, 0.06, 0.91, 15 );

	static UINT groups17[1] = { 38 };
	makeanimation( 17, groups17 , 1, _V( -1.982,0.768,-1.87-comshift), _V( 0.3587,0.93358,0), PI, 0.06, 0.91, 16 );

	static UINT groups18[1] = { 37 };
	makeanimation( 18, groups18 , 1, _V(-1.982,0.768,-3.823-comshift), _V( 0.3587,0.93358,0), -PI, 0.06, 0.91, 17 );

	static UINT groups19[1] = { 36 };
	makeanimation( 19, groups19 , 1, _V( -1.982,0.768,-1.87-comshift), _V( 0.3587,0.93358,0), PI, 0.06, 0.91, 18 );

/*	static UINT groups13[1] = { 51  };
	makeanimation( 13, groups13 , 1, _V( 9.049,-3.481,-2.112-comshift), _V( 0.3587,0.93358,0), 1.0*PI, 0.68+0.05, 0.87+0.05 );
*/
	static UINT groups21[5] = { 35,36,37,38,39 };
	makeanimation( 21, groups21 , 5, _V(-1.982,0.768,-2.112 -comshift), _V( 0.93358,-0.3587,0), 0.35*PI, 0.91, 1 );


	// bottom left foldy out thing
/*
	static UINT groups22[4] = { 42,43,44,45 };
	makeanimation( 22, groups22, 4, _V(-1.982,-0.768,-3.823-comshift), _V(-0.3587,0.93358,0), -0.5*PI, 0, 0.1+0.03 );

	static UINT groups23[5] = { 41,42,43,44,45 };
	makeanimation( 23, groups23 , 5, _V(-1.982,-0.768,-2.112 -comshift), _V( -0.3587,0.93358,0), 0.5*PI, 0.1+0.03, 0.2+0.03 );

	static UINT groups24[4] = { 42,43,44,45};
	makeanimation( 24, groups24 , 4, _V( -3.579,-1.381,-2.112-comshift), _V(-0.3587,0.93358,0 ), -0.5*PI, 0.2+0.03, 0.3+0.03 );

	static UINT groups25[3] = { 43,44,45 };
	makeanimation( 25, groups25 , 3, _V(-5.403,-2.081,-2.112 -comshift), _V(-0.3587,0.93358,0 ), -1.0*PI, 0.3+0.03, 0.5+0.03 );

	static UINT groups26[2] = { 44,45 };
	makeanimation( 26, groups26 , 2, _V(-7.226,-2.781,-2.112 -comshift), _V( -0.3587,0.93358,0), -1.0*PI, 0.5+0.03, 0.68+0.03 );

	static UINT groups27[1] = { 45  };
	makeanimation( 27, groups27 , 1, _V( -9.049,-3.481,-2.112-comshift), _V( -0.3587,0.93358,0), -1.0*PI, 0.68+0.03, 0.87+0.03 );

	static UINT groups28[5] = {41,42,43,44,45 };
	makeanimation( 28, groups28 , 5, _V(-1.982,-0.768,-2.112 -comshift), _V( -0.93358,-0.3587,0), -0.35*PI, 0.9+0.03, 0.97 );

	*/
	static UINT groups22[1] = { 41 };
	makeanimation( 22, groups22, 1, _V(-1.982,-0.768,-2.112-comshift), _V(-0.3587,0.93358,0), 0.5*PI, 0.02, 0.87 );

	static UINT groups23[1] = { 45 };
	makeanimation( 23, groups23 , 1, _V(-1.982,-0.768,-3.823-comshift ), _V(- 0.3587,0.93358,0), -PI, 0.02, 0.87, 22 );

	static UINT groups24[1] = { 44 };
	makeanimation( 24, groups24 , 1, _V( -1.982,-0.768,-1.87-comshift), _V(-0.3587,0.93358,0), PI, 0.02, 0.87, 23 );

	static UINT groups25[1] = { 43 };
	makeanimation( 25, groups25 , 1, _V(-1.982,-0.768,-3.823-comshift), _V( -0.3587,0.93358,0), -PI, 0.02, 0.87, 24 );

	static UINT groups26[1] = { 42 };
	makeanimation( 26, groups26 , 1, _V( -1.982,-0.768,-1.87-comshift), _V( -0.3587,0.93358,0), PI, 0.02, 0.87, 25 );

/*	static UINT groups13[1] = { 51  };
	makeanimation( 13, groups13 , 1, _V( 9.049,-3.481,-2.112-comshift), _V( 0.3587,0.93358,0), 1.0*PI, 0.68+0.05, 0.87+0.05 );
*/
	static UINT groups28[5] = { 41,42,43,44,45 };
	makeanimation( 28, groups28 , 5, _V(-1.982,-0.768,-2.112 -comshift), _V( -0.93358,-0.3587,0), -0.35*PI, 0.87, 0.97 );


}

void ATV::makeanimation( int i, UINT groups[], int n, VECTOR3 ref, VECTOR3 axis, double range , double start = 0, double end = 1, int parent ) {

	rot[i] = new MGROUP_ROTATE (
		mshid, // mesh index
		groups, n, // group list and # groups
		ref, // rotation reference point
		axis, // rotation axis
		(float) range// angular rotation range
	);
	if( parent >= 0 ) {
		animcomp[i] = AddAnimationComponent (anim[0], start, end, rot[i], animcomp[parent] );
	}
	else {
		animcomp[i] = AddAnimationComponent (anim[0], start, end, rot[i]);
	}

}

void ATV::VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) {
	setmainstage();
}
