/**
*
* $Id: fregat.cpp,v 1.5 2004/09/24 14:50:23 dh219 Exp $
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "fregat.h"

FREGAT::FREGAT (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hFregat = oapiLoadMeshGlobal ("fregat");
	tex = oapiRegisterExhaustTexture("Exhaust_atsme");

	/* set the propellant handle to NULL so I can check if I need to 
	/* create it in the SetMainStage() function. */
	ph_first=NULL; 

}


FREGAT::~FREGAT()
{

}


void FREGAT::TimeStep(  double simt )
{
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
		SetMainStage();	
	}
}


void FREGAT::SetClassCaps(  FILEHANDLE cfg )
{
	/* propellant must be set up in ClassCaps if it's to be set correctly at staging */
	if (ph_first==NULL)  
		ph_first = CreatePropellantResource(FIRST_PROP_MASS); 
}

void FREGAT::SaveState( FILEHANDLE scn )
{
	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void FREGAT::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while(oapiReadScenario_nextline (scn, line) ) {
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void FREGAT::SetMainStage( )
{

	const VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	SetSize (4);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( FREGAT_DRYMASS + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(17,17,1.5));
	SetCrossSections (_V(4,4,2.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 


	/* we're not using COMMON_MESH today, so draw our own */
	AddMesh(hFregat, &VECTOR_ZERO );
	

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	Draw();


	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0,0,-.2), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 2.0, 0.2,tex);
	SetAttControlsFregat();
	EnableTransponder(true);
}





/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */
int FREGAT::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	int result = 0;
	if(!fPressed && !KEYMOD_SHIFT(keystate) && !KEYMOD_CONTROL(keystate) && !KEYMOD_ALT(keystate))
	{
		switch(dwKey)
		{ 
		case OAPI_KEY_J:
			result = 1;

			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */

			DeployPayload();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			break;

		}
	}
	return result;
}



int FREGAT::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void FREGAT::SetAttControlsFregat()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}


/* CVE-lite function */
/* OPTIONAL */

/* Define the ParseCustomCVELString() function to receive inter-vessel messages! */

void FREGAT::ParseCustomCVELString(char *key, char *string) {

	/* if we receive a test message, let's just echo it to oapiDebugString() for now */

	if( !strcmp( key, "TEST" ) )
		sprintf(oapiDebugString(), "%s", string );

}