/**
*
* $Id: fregat.h,v 1.5 2004/09/24 14:50:23 dh219 Exp $
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "cvel.h"
#include <orbitersdk.h>
#include <cstdio>
#include <cmath>
#include <string>


const double ISP_FIRST_SL		=     270.0*G;
const double ISP_FIRST_VAC		=     327.0*G;
const double THRUST_FIRST_VAC	=    2000.0*G;
const double MAX_ATT_FIRST		=     250.0*G;
const double FIRST_PROP_MASS	=    5435.0;
const double FREGAT_MAX_BURN	=     877.0;
const double FREGAT_DRYMASS		=    1100.0;


/* make your own vessel class (named FREGAT here, for example)
   subclassed from COMMON_VESSEL_EX */

class FREGAT : public COMMON_VESSEL_EX {
public:
	FREGAT (OBJHANDLE hObj, int fmodel);
	~FREGAT();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );

	// callback from CVEL to pass through Custom CVEL strings
	void ParseCustomCVELString(char *key, char *string);

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsFregat();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[1];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hFregat;
	SURFHANDLE		  tex;

};




/* CVE-lite change ! */

/* CVEL no longer intecepts ovcVisualCreated() since we can't rotate meshes anyway.
/* you must make sure you have an ovcVisualCreated() if you previously relied on it.
*/

DLLCLBK void ovcVisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount) {
	FREGAT *fregat = (FREGAT*)vessel;
	fregat->SetMainStage(); // this is a good place to ask for the initial setup.
}



/* basic vessel class setup */


// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	FREGAT *fregat= new FREGAT( hvessel, flightmodel );
	return fregat;
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel) 
{
	if (vessel) delete (FREGAT*)vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	FREGAT *fregat = (FREGAT*)vessel;
	return fregat->ConsumeKey(  keystate );
}

// Keyboard interface handler
DLLCLBK int ovcConsumeBufferedKey (VESSEL *vessel, DWORD dwKey, bool fPressed, const char *keystate)
{
	FREGAT *fregat = (FREGAT*)vessel;
	return fregat->ConsumeBufferedKey( dwKey, fPressed, keystate );
}

DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	FREGAT *fregat = (FREGAT*)vessel;
	fregat->SetClassCaps(  cfg );
}

DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	FREGAT *fregat = (FREGAT*)vessel;
	fregat->TimeStep(  simt );
}

// Read status from scenario file
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)
{
	FREGAT *fregat = (FREGAT*)vessel;
	fregat->LoadState(  scn, vs );
}


// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	FREGAT *fregat= (FREGAT*)vessel;
	fregat->SaveState(  scn );

}


