// ==============================================================
// $Id: initguidance.h,v 1.4 2004/11/19 12:02:32 dh219 Exp $
// Initial Guidance Class by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

class IGUIDE {
public:
	IGUIDE(double psensitivity=0.05, double rsensitivity=0.05,double ysensitivity=0.15);
	void LaunchGuidance( VESSEL *v /*, double *pup , double *yleft, double *rleft, double simt */);
	void SetUserHeading(double head);
	double GetUserHeading();
	bool Active();
	void Active(bool a);

private:
	double initialheading;
	double initialgamma;
	double userheading;
	double ps;
	double rs;
	double ys;
	bool rotated;
	bool pitched;
	bool lockedin;
	bool active;

	void NullRoll(VESSEL *v);
	void NullPitch(VESSEL *v);
	void NullYaw(VESSEL *v);

};

IGUIDE::IGUIDE(double psensitivity, double rsensitivity, double ysensitivity) {
	initialheading = 0.0;
	initialgamma = 0.0;
	userheading = 90;
	rotated = false;
	pitched = false;
	lockedin = false;
	active = false;

	ps=psensitivity;
	rs=rsensitivity;
	ys=ysensitivity;
}



void IGUIDE::LaunchGuidance( VESSEL *v /*, double *pup , double *yleft, double *rleft, double simt */) {

	VESSELSTATUS2 vs;
	double diff ;
	double targetrate ;
	double pitchrate;
	double yawrate;
	double rate;

	double launchrotate;

	if( !active )
		return;
	
	vs.version = 2;
	vs.flag = 0;
	v->GetStatusEx(&vs);

	if( vs.status == 1 ) {
		initialheading = vs.surf_hdg * 180/PI;
		initialgamma = vs.arot.z * 180/PI;
		launchrotate =	userheading - initialheading;

//		sprintf( oapiDebugString(), "%2.2f %2.2f %2.2f", userheading, initialheading, launchrotate);

		return;
	}

	pitchrate = vs.vrot.x * 180/PI;

	yawrate = vs.vrot.y * 180/PI;


	launchrotate =	userheading - initialheading;
/*	launchrotate = 82.0; */


	if( launchrotate < -180.0 )
		launchrotate += 360.0;
	else if( launchrotate > 180.0 )
		launchrotate -= 360.0;



	double alt = v->GetAltitude();
	double pitch = v->GetPitch() * 180/PI;
	double heading;
	oapiGetHeading( v->GetHandle(), &heading);
	heading *= 180/PI;
	double roll = v->GetBank() * 180/PI;

	double gamma = vs.arot.z * 180/PI;


	if( alt > 100.0 && alt < 16000.0 && vs.status == 0 ) {


		if( !rotated && fabs(v->GetPitch() * 180/PI) > 88.0 ) {
			// roll to heading 

			double target = initialgamma - launchrotate;
			if( target > 180.0 )
				target -= 360.0;
			else if( target < -180.0 )
				target += 360.0;
					
			diff = vs.arot.z*180/PI - target;
			if( diff > 180.0 )
				diff -= 360.0;
			else if( diff < -180.0 )
				diff += 360.0;

			targetrate = diff/-1.0;
			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vs.vrot.z * 180/PI;

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
					v->IncThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,rs);
					v->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0.0 );
				}
				else {
					v->IncThrusterGroupLevel(THGROUP_ATT_BANKLEFT,rs);
					v->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0.0 );
				}
			}
			else {
				v->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0.0 );
			}
		
			if( fabs(diff) < 1.0 /*&& fabs(rate) < 0.5*/ ) {
//				v->ActivateNavmode(NAVMODE_KILLROT);
				rotated = true;
			}

		
		}

		else if( rotated && !pitched && !v->GetNavmodeState(NAVMODE_KILLROT) ) {
			// pitch to elevation

			double target = 80.0;
			double pitch = v->GetPitch() * 180/PI;
			if( fabs(v->GetBank()) > PI/2.0 )
				pitch = 180-pitch;

					
			diff =  pitch - target;

			targetrate = diff/-2.0;
			if( targetrate > 1.5 )
				targetrate = 1.5;
			else if( targetrate < -1.5 )
				targetrate = -1.5;
			
			
			rate = vs.vrot.x * 180/PI;

			if( fabs( rate-targetrate ) > 0.05 ) {
				NullYaw(v);
				NullRoll(v);
				if( rate < targetrate ) {
					v->IncThrusterGroupLevel(THGROUP_ATT_PITCHUP,ps);
					v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
				}
				else {
					v->IncThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,ps);
					v->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0.0 );
				}
			}
			else {
				v->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
			}

			if( fabs(diff) < 1.0  ) {
//				v->ActivateNavmode(NAVMODE_KILLROT);
				pitched = true;
			}



//			sprintf( oapiDebugString(), "pitching: pitch %2.2f rate: %2.2f target: %2.2f", pitch, rate, targetrate);

		}
/*		else if( pitched  && !lockedin) {
			// yaw to alignment

			double target = 0.0;
			double bank = v->GetBank() * 180/PI;
					
			diff =  bank - target;

			targetrate = diff/-2.0;
			if( targetrate > 0.1 )
				targetrate = 0.1;
			else if( targetrate < -0.1 )
				targetrate = -0.1;
			
			
			rate = vs.vrot.x * 180/PI;

			if( fabs( rate-targetrate ) > 0.05 ) {
				NullRoll(v);
				NullPitch(v);

				if( rate < targetrate ) {
					v->IncThrusterGroupLevel(THGROUP_ATT_YAWLEFT,ys);
					v->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0.0 );
				}
				else {
					v->IncThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,ys);
					v->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0.0 );
				}
			}
			else {
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0.0 );
			}

			if( fabs(diff) < 1.0  ) {
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0.0 );
				v->ActivateNavmode(NAVMODE_KILLROT);
				lockedin = true;
			}

//			sprintf( oapiDebugString(), "yawing: rate: %2.2f target: %2.2f", rate, targetrate);
	
		}	

*/	
	
	}
	else if( v->GetAltitude() > 16000.0 /*&& lockedin*/ ) {
		v->DeactivateNavmode(NAVMODE_KILLROT);
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0.0 );
				v->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0.0 );
		lockedin = false;
		active = false;
	}


	return;
}


void IGUIDE::SetUserHeading(double head) {
	userheading = head;
}


void IGUIDE::NullRoll(VESSEL *v) {

	VESSELSTATUS vs;
	v->GetStatus(vs);

	double rate = vs.vrot.z / 2.0;

	if( fabs( rate ) > 0.05 ) {
		if( rate < 0.0) {
			v->IncThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,rs);
			v->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0.0 );
		}
		else {
			v->IncThrusterGroupLevel(THGROUP_ATT_BANKLEFT,rs);
			v->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0.0 );
		}
	}
	else {
		v->SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0.0 );
		v->SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0.0 );
	}

	
}
void IGUIDE::NullPitch(VESSEL *v) {

	VESSELSTATUS vs;
	v->GetStatus(vs);

	double rate = vs.vrot.x / 2.0;

	if( fabs( rate ) > 0.05 ) {
		if( rate < 0.0) {
			v->IncThrusterGroupLevel(THGROUP_ATT_PITCHUP,rs);
			v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
		}
		else {
			v->IncThrusterGroupLevel(THGROUP_ATT_PITCHUP,rs);
			v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
		}
	}
	else {
		v->SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0.0 );
		v->SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0.0 );
	}

}
void IGUIDE::NullYaw(VESSEL *v) {


}

void IGUIDE::Active(bool a) {
	active = a;
}

bool IGUIDE::Active() {
	return active;
}

double IGUIDE::GetUserHeading() {
	return userheading;
}
